/***********************************************
/gen auto by zuitools
***********************************************/
#include "mainActivity.h"

/*TAG:GlobalVariable全局变量*/
static ZKListView* mMsgListViewPtr;
static ZKRadioGroup* mTcpPortRadioGroupPtr;
static ZKTextView* mTcpPortTextViewPtr;
static ZKTextView* mTextView5Ptr;
static ZKTextView* mtvDatetimePtr;
static ZKTextView* mtvTcpPtr;
static ZKTextView* mtvNetPtr;
static ZKButton* mBtnRestartPtr;
static ZKButton* mBtnSoftVerPtr;
static ZKQRCode* mQRCode1Ptr;
static ZKTextView* mVersionPtr;
static ZKTextView* mGatewayPtr;
static ZKTextView* mTextView11Ptr;
static ZKButton* mBtnNetSettingPtr;
static ZKTextView* mTextView8Ptr;
static ZKTextView* mServerIPMsgPtr;
static ZKButton* mBtnServerIPConfirmPtr;
static ZKTextView* mTextView6Ptr;
static ZKEditText* mEditTextServerIPPtr;
static ZKWindow* mWindowServerIPPtr;
static ZKTextView* mTextView7Ptr;
static ZKTextView* mTextView3Ptr;
static ZKTextView* mTextView12Ptr;
static ZKButton* mSettingPtr;
static ZKTextView* mTextPwdInfoPtr;
static ZKButton* mBtnPwdConfirmPtr;
static ZKTextView* mTextView2Ptr;
static ZKEditText* mEditTextPwdPtr;
static ZKWindow* mWindowPwdPtr;
static ZKTextView* mNetMaskPtr;
static ZKTextView* mDeviceIpPtr;
static ZKTextView* mServerIPPtr;
static ZKTextView* mDeviceCodePtr;
static ZKTextView* mDeviceMacPtr;
static ZKTextView* mTextView4Ptr;
static ZKTextView* mTextView10Ptr;
static mainActivity* mActivityPtr;

/*register activity*/
REGISTER_ACTIVITY(mainActivity);

typedef struct {
	int id; // 定时器ID ， 不能重复
	int time; // 定时器  时间间隔  单位 毫秒
}S_ACTIVITY_TIMEER;

#include "logic/mainLogic.cc"

/***********/
typedef struct {
    int id;
    const char *pApp;
} SAppInfo;

/**
 *点击跳转window
 */
static SAppInfo sAppInfoTab[] = {
//  { ID_MAIN_TEXT, "TextViewActivity" },
};

/***************/
typedef bool (*ButtonCallback)(ZKButton *pButton);
/**
 * button onclick表
 */
typedef struct {
    int id;
    ButtonCallback callback;
}S_ButtonCallback;

/*TAG:ButtonCallbackTab按键映射表*/
static S_ButtonCallback sButtonCallbackTab[] = {
    ID_MAIN_BtnRestart, onButtonClick_BtnRestart,
    ID_MAIN_BtnSoftVer, onButtonClick_BtnSoftVer,
    ID_MAIN_BtnNetSetting, onButtonClick_BtnNetSetting,
    ID_MAIN_BtnServerIPConfirm, onButtonClick_BtnServerIPConfirm,
    ID_MAIN_Setting, onButtonClick_Setting,
    ID_MAIN_BtnPwdConfirm, onButtonClick_BtnPwdConfirm,
};
/***************/


typedef void (*SeekBarCallback)(ZKSeekBar *pSeekBar, int progress);
typedef struct {
    int id;
    SeekBarCallback callback;
}S_ZKSeekBarCallback;
/*TAG:SeekBarCallbackTab*/
static S_ZKSeekBarCallback SZKSeekBarCallbackTab[] = {
};


typedef int (*ListViewGetItemCountCallback)(const ZKListView *pListView);
typedef void (*ListViewobtainListItemDataCallback)(ZKListView *pListView,ZKListView::ZKListItem *pListItem, int index);
typedef void (*ListViewonItemClickCallback)(ZKListView *pListView, int index, int id);
typedef struct {
    int id;
    ListViewGetItemCountCallback getListItemCountCallback;
    ListViewobtainListItemDataCallback obtainListItemDataCallback;
    ListViewonItemClickCallback onItemClickCallback;
}S_ListViewFunctionsCallback;
/*TAG:ListViewFunctionsCallback*/
static S_ListViewFunctionsCallback SListViewFunctionsCallbackTab[] = {
    ID_MAIN_MsgListView, getListItemCount_MsgListView, obtainListItemData_MsgListView, onListItemClick_MsgListView,
};


typedef void (*SlideWindowItemClickCallback)(ZKSlideWindow *pSlideWindow, int index);
typedef struct {
    int id;
    SlideWindowItemClickCallback onSlideItemClickCallback;
}S_SlideWindowItemClickCallback;
/*TAG:SlideWindowFunctionsCallbackTab*/
static S_SlideWindowItemClickCallback SSlideWindowItemClickCallbackTab[] = {
};


typedef void (*EditTextInputCallback)(const std::string &text);
typedef struct {
    int id;
    EditTextInputCallback onEditTextChangedCallback;
}S_EditTextInputCallback;
/*TAG:EditTextInputCallback*/
static S_EditTextInputCallback SEditTextInputCallbackTab[] = {
    ID_MAIN_EditTextServerIP, onEditTextChanged_EditTextServerIP,
    ID_MAIN_EditTextPwd, onEditTextChanged_EditTextPwd,
};

typedef void (*VideoViewCallback)(ZKVideoView *pVideoView, int msg);
typedef struct {
    int id; //VideoView ID
    bool loop; // 是否是轮播类型
    int defaultvolume;//轮播类型时,默认视频音量
    VideoViewCallback onVideoViewCallback;
}S_VideoViewCallback;
/*TAG:VideoViewCallback*/
static S_VideoViewCallback SVideoViewCallbackTab[] = {
};


typedef void (*RadioGroupCallback)(ZKRadioGroup*, int);
typedef struct {
  int id;
  RadioGroupCallback onCheckedChanged;
}S_RadioGroupCallback;
/*TAG:RadioGroupCallbackTab*/
static S_RadioGroupCallback SRadioGroupCallbackTab[] = {
    ID_MAIN_TcpPortRadioGroup, onCheckedChanged_TcpPortRadioGroup,
};

mainActivity::mainActivity() {
	//todo add init code here
	mVideoLoopIndex = -1;
	mVideoLoopErrorCount = 0;
}

mainActivity::~mainActivity() {
  //todo add init file here
  // 退出应用时需要反注册
    EASYUICONTEXT->unregisterGlobalTouchListener(this);
    onUI_quit();
    unregisterProtocolDataUpdateListener(onProtocolDataUpdate);
    mMsgListViewPtr = NULL;
    mTcpPortRadioGroupPtr = NULL;
    mTcpPortTextViewPtr = NULL;
    mTextView5Ptr = NULL;
    mtvDatetimePtr = NULL;
    mtvTcpPtr = NULL;
    mtvNetPtr = NULL;
    mBtnRestartPtr = NULL;
    mBtnSoftVerPtr = NULL;
    mQRCode1Ptr = NULL;
    mVersionPtr = NULL;
    mGatewayPtr = NULL;
    mTextView11Ptr = NULL;
    mBtnNetSettingPtr = NULL;
    mTextView8Ptr = NULL;
    mServerIPMsgPtr = NULL;
    mBtnServerIPConfirmPtr = NULL;
    mTextView6Ptr = NULL;
    mEditTextServerIPPtr = NULL;
    mWindowServerIPPtr = NULL;
    mTextView7Ptr = NULL;
    mTextView3Ptr = NULL;
    mTextView12Ptr = NULL;
    mSettingPtr = NULL;
    mTextPwdInfoPtr = NULL;
    mBtnPwdConfirmPtr = NULL;
    mTextView2Ptr = NULL;
    mEditTextPwdPtr = NULL;
    mWindowPwdPtr = NULL;
    mNetMaskPtr = NULL;
    mDeviceIpPtr = NULL;
    mServerIPPtr = NULL;
    mDeviceCodePtr = NULL;
    mDeviceMacPtr = NULL;
    mTextView4Ptr = NULL;
    mTextView10Ptr = NULL;
    mActivityPtr = NULL;
}

const char* mainActivity::getAppName() const{
	return "main.ftu";
}

//TAG:onCreate
void mainActivity::onCreate() {
	Activity::onCreate();
    mMsgListViewPtr = (ZKListView*)findControlByID(ID_MAIN_MsgListView);if(mMsgListViewPtr!= NULL){mMsgListViewPtr->setListAdapter(this);mMsgListViewPtr->setItemClickListener(this);}
    mTcpPortRadioGroupPtr = (ZKRadioGroup*)findControlByID(ID_MAIN_TcpPortRadioGroup);if(mTcpPortRadioGroupPtr!= NULL){mTcpPortRadioGroupPtr->setCheckedChangeListener(this);}
    mTcpPortTextViewPtr = (ZKTextView*)findControlByID(ID_MAIN_TcpPortTextView);
    mTextView5Ptr = (ZKTextView*)findControlByID(ID_MAIN_TextView5);
    mtvDatetimePtr = (ZKTextView*)findControlByID(ID_MAIN_tvDatetime);
    mtvTcpPtr = (ZKTextView*)findControlByID(ID_MAIN_tvTcp);
    mtvNetPtr = (ZKTextView*)findControlByID(ID_MAIN_tvNet);
    mBtnRestartPtr = (ZKButton*)findControlByID(ID_MAIN_BtnRestart);
    mBtnSoftVerPtr = (ZKButton*)findControlByID(ID_MAIN_BtnSoftVer);
    mQRCode1Ptr = (ZKQRCode*)findControlByID(ID_MAIN_QRCode1);
    mVersionPtr = (ZKTextView*)findControlByID(ID_MAIN_Version);
    mGatewayPtr = (ZKTextView*)findControlByID(ID_MAIN_Gateway);
    mTextView11Ptr = (ZKTextView*)findControlByID(ID_MAIN_TextView11);
    mBtnNetSettingPtr = (ZKButton*)findControlByID(ID_MAIN_BtnNetSetting);
    mTextView8Ptr = (ZKTextView*)findControlByID(ID_MAIN_TextView8);
    mServerIPMsgPtr = (ZKTextView*)findControlByID(ID_MAIN_ServerIPMsg);
    mBtnServerIPConfirmPtr = (ZKButton*)findControlByID(ID_MAIN_BtnServerIPConfirm);
    mTextView6Ptr = (ZKTextView*)findControlByID(ID_MAIN_TextView6);
    mEditTextServerIPPtr = (ZKEditText*)findControlByID(ID_MAIN_EditTextServerIP);if(mEditTextServerIPPtr!= NULL){mEditTextServerIPPtr->setTextChangeListener(this);}
    mWindowServerIPPtr = (ZKWindow*)findControlByID(ID_MAIN_WindowServerIP);
    mTextView7Ptr = (ZKTextView*)findControlByID(ID_MAIN_TextView7);
    mTextView3Ptr = (ZKTextView*)findControlByID(ID_MAIN_TextView3);
    mTextView12Ptr = (ZKTextView*)findControlByID(ID_MAIN_TextView12);
    mSettingPtr = (ZKButton*)findControlByID(ID_MAIN_Setting);
    mTextPwdInfoPtr = (ZKTextView*)findControlByID(ID_MAIN_TextPwdInfo);
    mBtnPwdConfirmPtr = (ZKButton*)findControlByID(ID_MAIN_BtnPwdConfirm);
    mTextView2Ptr = (ZKTextView*)findControlByID(ID_MAIN_TextView2);
    mEditTextPwdPtr = (ZKEditText*)findControlByID(ID_MAIN_EditTextPwd);if(mEditTextPwdPtr!= NULL){mEditTextPwdPtr->setTextChangeListener(this);}
    mWindowPwdPtr = (ZKWindow*)findControlByID(ID_MAIN_WindowPwd);
    mNetMaskPtr = (ZKTextView*)findControlByID(ID_MAIN_NetMask);
    mDeviceIpPtr = (ZKTextView*)findControlByID(ID_MAIN_DeviceIp);
    mServerIPPtr = (ZKTextView*)findControlByID(ID_MAIN_ServerIP);
    mDeviceCodePtr = (ZKTextView*)findControlByID(ID_MAIN_DeviceCode);
    mDeviceMacPtr = (ZKTextView*)findControlByID(ID_MAIN_DeviceMac);
    mTextView4Ptr = (ZKTextView*)findControlByID(ID_MAIN_TextView4);
    mTextView10Ptr = (ZKTextView*)findControlByID(ID_MAIN_TextView10);
	mActivityPtr = this;
	onUI_init();
    registerProtocolDataUpdateListener(onProtocolDataUpdate); 
    rigesterActivityTimer();
}

void mainActivity::onClick(ZKBase *pBase) {
	//TODO: add widget onClik code 
    int buttonTablen = sizeof(sButtonCallbackTab) / sizeof(S_ButtonCallback);
    for (int i = 0; i < buttonTablen; ++i) {
        if (sButtonCallbackTab[i].id == pBase->getID()) {
            if (sButtonCallbackTab[i].callback((ZKButton*)pBase)) {
            	return;
            }
            break;
        }
    }


    int len = sizeof(sAppInfoTab) / sizeof(sAppInfoTab[0]);
    for (int i = 0; i < len; ++i) {
        if (sAppInfoTab[i].id == pBase->getID()) {
            EASYUICONTEXT->openActivity(sAppInfoTab[i].pApp);
            return;
        }
    }

	Activity::onClick(pBase);
}

void mainActivity::onResume() {
	Activity::onResume();
	EASYUICONTEXT->registerGlobalTouchListener(this);
	startVideoLoopPlayback();
	onUI_show();
}

void mainActivity::onPause() {
	Activity::onPause();
	EASYUICONTEXT->unregisterGlobalTouchListener(this);
	stopVideoLoopPlayback();
	onUI_hide();
}

void mainActivity::onIntent(const Intent *intentPtr) {
	Activity::onIntent(intentPtr);
	onUI_intent(intentPtr);
}

bool mainActivity::onTimer(int id) {
	return onUI_Timer(id);
}

void mainActivity::onProgressChanged(ZKSeekBar *pSeekBar, int progress){

    int seekBarTablen = sizeof(SZKSeekBarCallbackTab) / sizeof(S_ZKSeekBarCallback);
    for (int i = 0; i < seekBarTablen; ++i) {
        if (SZKSeekBarCallbackTab[i].id == pSeekBar->getID()) {
            SZKSeekBarCallbackTab[i].callback(pSeekBar, progress);
            break;
        }
    }
}

int mainActivity::getListItemCount(const ZKListView *pListView) const{
    int tablen = sizeof(SListViewFunctionsCallbackTab) / sizeof(S_ListViewFunctionsCallback);
    for (int i = 0; i < tablen; ++i) {
        if (SListViewFunctionsCallbackTab[i].id == pListView->getID()) {
            return SListViewFunctionsCallbackTab[i].getListItemCountCallback(pListView);
            break;
        }
    }
    return 0;
}

void mainActivity::obtainListItemData(ZKListView *pListView,ZKListView::ZKListItem *pListItem, int index){
    int tablen = sizeof(SListViewFunctionsCallbackTab) / sizeof(S_ListViewFunctionsCallback);
    for (int i = 0; i < tablen; ++i) {
        if (SListViewFunctionsCallbackTab[i].id == pListView->getID()) {
            SListViewFunctionsCallbackTab[i].obtainListItemDataCallback(pListView, pListItem, index);
            break;
        }
    }
}

void mainActivity::onItemClick(ZKListView *pListView, int index, int id){
    int tablen = sizeof(SListViewFunctionsCallbackTab) / sizeof(S_ListViewFunctionsCallback);
    for (int i = 0; i < tablen; ++i) {
        if (SListViewFunctionsCallbackTab[i].id == pListView->getID()) {
            SListViewFunctionsCallbackTab[i].onItemClickCallback(pListView, index, id);
            break;
        }
    }
}

void mainActivity::onSlideItemClick(ZKSlideWindow *pSlideWindow, int index) {
    int tablen = sizeof(SSlideWindowItemClickCallbackTab) / sizeof(S_SlideWindowItemClickCallback);
    for (int i = 0; i < tablen; ++i) {
        if (SSlideWindowItemClickCallbackTab[i].id == pSlideWindow->getID()) {
            SSlideWindowItemClickCallbackTab[i].onSlideItemClickCallback(pSlideWindow, index);
            break;
        }
    }
}

bool mainActivity::onTouchEvent(const MotionEvent &ev) {
    return onmainActivityTouchEvent(ev);
}

void mainActivity::onTextChanged(ZKTextView *pTextView, const std::string &text) {
    int tablen = sizeof(SEditTextInputCallbackTab) / sizeof(S_EditTextInputCallback);
    for (int i = 0; i < tablen; ++i) {
        if (SEditTextInputCallbackTab[i].id == pTextView->getID()) {
            SEditTextInputCallbackTab[i].onEditTextChangedCallback(text);
            break;
        }
    }
}

void mainActivity::rigesterActivityTimer() {
    int tablen = sizeof(REGISTER_ACTIVITY_TIMER_TAB) / sizeof(S_ACTIVITY_TIMEER);
    for (int i = 0; i < tablen; ++i) {
        S_ACTIVITY_TIMEER temp = REGISTER_ACTIVITY_TIMER_TAB[i];
        registerTimer(temp.id, temp.time);
    }
}


void mainActivity::onVideoPlayerMessage(ZKVideoView *pVideoView, int msg) {
    int tablen = sizeof(SVideoViewCallbackTab) / sizeof(S_VideoViewCallback);
    for (int i = 0; i < tablen; ++i) {
        if (SVideoViewCallbackTab[i].id == pVideoView->getID()) {
        	if (SVideoViewCallbackTab[i].loop) {
                //循环播放
        		videoLoopPlayback(pVideoView, msg, i);
        	} else if (SVideoViewCallbackTab[i].onVideoViewCallback != NULL){
        	    SVideoViewCallbackTab[i].onVideoViewCallback(pVideoView, msg);
        	}
            break;
        }
    }
}

void mainActivity::videoLoopPlayback(ZKVideoView *pVideoView, int msg, size_t callbackTabIndex) {

	switch (msg) {
	case ZKVideoView::E_MSGTYPE_VIDEO_PLAY_STARTED:
		LOGD("ZKVideoView::E_MSGTYPE_VIDEO_PLAY_STARTED\n");
    if (callbackTabIndex >= (sizeof(SVideoViewCallbackTab)/sizeof(S_VideoViewCallback))) {
      break;
    }
		pVideoView->setVolume(SVideoViewCallbackTab[callbackTabIndex].defaultvolume / 10.0);
		mVideoLoopErrorCount = 0;
		break;
	case ZKVideoView::E_MSGTYPE_VIDEO_PLAY_ERROR:
		/**错误处理 */
		++mVideoLoopErrorCount;
		if (mVideoLoopErrorCount > 100) {
			LOGD("video loop error counts > 100, quit loop playback !");
            break;
		} //不用break, 继续尝试播放下一个
	case ZKVideoView::E_MSGTYPE_VIDEO_PLAY_COMPLETED:
		LOGD("ZKVideoView::E_MSGTYPE_VIDEO_PLAY_COMPLETED\n");
        std::vector<std::string> videolist;
        std::string fileName(getAppName());
        if (fileName.size() < 4) {
             LOGD("getAppName size < 4, ignore!");
             break;
        }
        fileName = fileName.substr(0, fileName.length() - 4) + "_video_list.txt";
        fileName = "/mnt/extsd/" + fileName;
        if (!parseVideoFileList(fileName.c_str(), videolist)) {
            LOGD("parseVideoFileList failed !");
		    break;
        }
		if (pVideoView && !videolist.empty()) {
			mVideoLoopIndex = (mVideoLoopIndex + 1) % videolist.size();
			pVideoView->play(videolist[mVideoLoopIndex].c_str());
		}
		break;
	}
}

void mainActivity::startVideoLoopPlayback() {
    int tablen = sizeof(SVideoViewCallbackTab) / sizeof(S_VideoViewCallback);
    for (int i = 0; i < tablen; ++i) {
    	if (SVideoViewCallbackTab[i].loop) {
    		ZKVideoView* videoView = (ZKVideoView*)findControlByID(SVideoViewCallbackTab[i].id);
    		if (!videoView) {
    			return;
    		}
    		//循环播放
    		videoLoopPlayback(videoView, ZKVideoView::E_MSGTYPE_VIDEO_PLAY_COMPLETED, i);
    		return;
    	}
    }
}

void mainActivity::stopVideoLoopPlayback() {
    int tablen = sizeof(SVideoViewCallbackTab) / sizeof(S_VideoViewCallback);
    for (int i = 0; i < tablen; ++i) {
    	if (SVideoViewCallbackTab[i].loop) {
    		ZKVideoView* videoView = (ZKVideoView*)findControlByID(SVideoViewCallbackTab[i].id);
    		if (!videoView) {
    			return;
    		}
    		if (videoView->isPlaying()) {
    		    videoView->stop();
    		}
    		return;
    	}
    }
}

bool mainActivity::parseVideoFileList(const char *pFileListPath, std::vector<string>& mediaFileList) {
	mediaFileList.clear();
	if (NULL == pFileListPath || 0 == strlen(pFileListPath)) {
        LOGD("video file list is null!");
		return false;
	}

	ifstream is(pFileListPath, ios_base::in);
	if (!is.is_open()) {
		LOGD("cann't open file %s \n", pFileListPath);
		return false;
	}
	char tmp[1024] = {0};
	while (is.getline(tmp, sizeof(tmp))) {
		string str = tmp;
		removeCharFromString(str, '\"');
		removeCharFromString(str, '\r');
		removeCharFromString(str, '\n');
		if (str.size() > 1) {
     		mediaFileList.push_back(str.c_str());
		}
	}
  LOGD("(f:%s, l:%d) parse fileList[%s], get [%d]files", __FUNCTION__,
      __LINE__, pFileListPath, int(mediaFileList.size()));
  for (std::vector<string>::size_type i = 0; i < mediaFileList.size(); i++) {
    LOGD("file[%u]:[%s]", ::size_t(i), mediaFileList[i].c_str());
  }
	is.close();

	return true;
}

int mainActivity::removeCharFromString(string& nString, char c) {
    string::size_type   pos;
    while(1) {
        pos = nString.find(c);
        if(pos != string::npos) {
            nString.erase(pos, 1);
        } else {
            break;
        }
    }
    return (int)nString.size();
}

void mainActivity::registerUserTimer(int id, int time) {
	registerTimer(id, time);
}

void mainActivity::unregisterUserTimer(int id) {
	unregisterTimer(id);
}

void mainActivity::resetUserTimer(int id, int time) {
	resetTimer(id, time);
}
void mainActivity::onCheckedChanged(ZKRadioGroup* pRadioGroup, int checkedID) {
    int tablen = sizeof(SRadioGroupCallbackTab) / sizeof(S_RadioGroupCallback);
    for (int i = 0; i < tablen; ++i) {
        if (SRadioGroupCallbackTab[i].id == pRadioGroup->getID()) {
        	SRadioGroupCallbackTab[i].onCheckedChanged(pRadioGroup, checkedID);
            break;
        }
    }
}